@@ -1347,11 +1347,12 @@ access_can_touch_variable (tree ref, tree alias, HOST_WIDE_INT offset,
 	   && flag_strict_aliasing
 	   && TREE_CODE (ref) != INDIRECT_REF
 	   && !MTAG_P (alias)
+	   && !var_ann (alias)->is_heapvar
 	   && !POINTER_TYPE_P (TREE_TYPE (alias))
 	   && offsetgtz
 	   && DECL_SIZE (alias)
 	   && TREE_CODE (DECL_SIZE (alias)) == INTEGER_CST
-	   && uoffset > TREE_INT_CST_LOW (DECL_SIZE (alias)))
+	   && uoffset >= TREE_INT_CST_LOW (DECL_SIZE (alias)))
     {
 #ifdef ACCESS_DEBUGGING
       fprintf (stderr, "Access to ");
@@ -2059,6 +2060,17 @@ get_modify_stmt_operands (tree stmt, tree expr)
      We used to distinguish between preserving and killing definitions.
      We always emit preserving definitions now.  */
   get_expr_operands (stmt, &GIMPLE_STMT_OPERAND (expr, 0), opf_def);
+
+  /* Make sure the return value is addressable in case of NRV.  */
+  if (TREE_CODE (GIMPLE_STMT_OPERAND (expr, 1)) == CALL_EXPR
+      && CALL_EXPR_RETURN_SLOT_OPT (GIMPLE_STMT_OPERAND (expr, 1))
+      && TREE_ADDRESSABLE (TREE_TYPE (GIMPLE_STMT_OPERAND (expr, 0))))
+    {
+      tree t = get_base_address (GIMPLE_STMT_OPERAND (expr, 0));
+      stmt_ann_t s_ann = stmt_ann (stmt);
+      if (t && DECL_P (t) && s_ann)
+	add_to_addressable_set (t, &s_ann->addresses_taken);
+    }
 }
 
 
